/*
  ftNano
  https://www.blafusel.de

  Steuert einen Kran auf einer horizontalen Achse und bewegt den Arm
  Motoren mit Encoder
  Waehrend der Motorbewegungen blinkt eine LED

  Faehrt beide Motoren beim Start an die Nullstellung (Taster -  gegen Masse via Treiber mit Pull-Up)


  Moegliche Erweiterungen:
    - IRQ-Erkennung der Null-Anschlaege mit "Not"-Halt
    - Taster an den Bewegungsenden
    - Not-Aus via Software oder Hardware
    - gleichzeitiges Verfahren beider Motoren
    - Funktionen fuer Bewegungen bis Count-Wert
    - Elektromagnet

*/

// #define M1_1  3
// #define M1_2  5  
// #define M2_1  2
// #define M2_2  4
#define M3_1  6
#define M3_2  9
// #define M4_1  7
// #define M4_2  8
#define M5_1  10
#define M5_2  11
// #define M6_1  12
// #define M6_2  13

const int16_t maxH = 6450;     // max. impulses horizontally
const int16_t maxV = 2200;     // max. impulses vertically
volatile int8_t dirH = 0;
volatile int8_t dirV = 0;
volatile int16_t cntH = 0;    // must be volatile because changing in IRQ-routine
volatile int16_t cntV = 0;    // must be volatile because changing in IRQ-routine
const uint8_t ledLamp = A3;     // pin LED
unsigned long previousMillis = 0;  // will store last time LED was updated
const long interval = 300;    // interval at which to blink (milliseconds)
const uint16_t pause = 1000;     // wait ms between moving

void blinkLamp ()
{
  unsigned long currentMillis = millis();
  if ((!digitalRead(ledLamp) && (currentMillis - previousMillis >= interval / 3)) || (digitalRead(ledLamp) && (currentMillis - previousMillis >= interval)))    // AN: angegebene Zeit; AUS: Zeit/2
  {
    previousMillis = currentMillis;   
    digitalWrite (ledLamp, !digitalRead(ledLamp));
  }
}

void blinkLampOff ()
{
  digitalWrite(ledLamp, LOW);
}

void HmotorStop ()
{
  digitalWrite (M5_1, HIGH);   
  digitalWrite (M5_2, HIGH);  
  blinkLampOff ();
}

void HmotorCounterClockW ()
{
  dirH = -1;
  digitalWrite (M5_1, LOW);
  digitalWrite (M5_2, HIGH);
}

void HmotorClockW ()
{
  dirH = 1;
  digitalWrite (M5_1, HIGH);
  digitalWrite (M5_2, LOW);
}

void VmotorStop ()
{
  digitalWrite (M3_1, LOW);   
  digitalWrite (M3_2, LOW);  
  blinkLampOff ();
}

void VmotorCounterClockW ()
{
  dirV = 1;
  digitalWrite (M3_1, LOW);
  digitalWrite (M3_2, HIGH);
}

void VmotorClockW ()
{
  dirV = -1;
  digitalWrite (M3_1, HIGH);
  digitalWrite (M3_2, LOW);
}

void moveHStart ()
{
  HmotorCounterClockW (); 
  while (!digitalRead(A0))
  {
    blinkLamp();
  }
  HmotorStop ();
  cntH = 0;
}

void moveVStart ()
{
  VmotorClockW (); 
  while (!digitalRead(A1))
  {
    blinkLamp();
  }
  VmotorStop ();
  cntV = 0;
}

// count interrupts
void countH() {
  cntH += dirH;
}

// count interrupts
void countV() {
  cntV += dirV;
}

void setup() {
  pinMode (M5_1, OUTPUT);   // motor horizontal
  pinMode (M5_2, OUTPUT);
  pinMode (M3_1, OUTPUT);   // motor vertical
  pinMode (M3_2, OUTPUT);
  pinMode (ledLamp, OUTPUT);   // lamp
  digitalWrite (ledLamp, LOW);   
  pinMode (A0, INPUT);      // switch horizontal
  pinMode (A1, INPUT);      // switch vertical
  pinMode (2, INPUT);   // impulses motor h
  pinMode (3, INPUT);   // impulses motor v
  attachInterrupt(digitalPinToInterrupt(2), countH, CHANGE);
  attachInterrupt(digitalPinToInterrupt(3), countV, CHANGE);
}   

void loop() 
{
  // Home
  HmotorStop ();
  VmotorStop ();
  moveVStart ();
  moveHStart ();

  delay(pause);

  // move a little dir end
  HmotorClockW ();
  while (cntH < 1000)
  {
    blinkLamp();
  }
  HmotorStop ();

  delay(pause);

  // Arm down
  VmotorCounterClockW ();  
  while (cntV < 1900)
  {
    blinkLamp();
  }
  VmotorStop ();

  delay(pause);

  // Arm up
  VmotorClockW ();  
  while (cntV > 300)
  {
    blinkLamp();
  }
  VmotorStop ();

  delay(pause);

  // move more dir end
  HmotorClockW ();
  while (cntH < maxH-50)
  {
    blinkLamp();
  }
  HmotorStop ();

  delay(pause);

  // Arm down
  VmotorCounterClockW ();  
  while (cntV < 1900)
  {
    blinkLamp();
  }
  VmotorStop ();

  delay(pause);

  // Arm up
  VmotorClockW ();  
  while (cntV > 300)
  {
    blinkLamp();
  }
  VmotorStop ();

  delay(pause);

  // move dir start
  HmotorCounterClockW ();
  while (cntH > 2000)
  {
    blinkLamp();
  }
  HmotorStop ();

  delay(pause);

  // Arm down
  VmotorCounterClockW ();  
  while (cntV < 600)
  {
    blinkLamp();
  }
  VmotorStop ();

  delay (10000);
}
